/***************************************************************************
 Copyright (C) 2007 Christoph Reichenbach <reichenb@colorado.edu>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public Licence as
 published by the Free Software Foundaton; either version 2 of the
 Licence, or (at your option) any later version.

 It is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 merchantability or fitness for a particular purpose. See the
 GNU General Public Licence for more details.

 You should have received a copy of the GNU General Public Licence
 along with this program; see the file COPYING. If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.

***************************************************************************/

package edu.colorado.cs;

import java.io.InputStream;

public class Game extends AdvParser implements ReaderInterface, RoomReader
{
	protected GameWorld world = new GameWorld(this, this);
	protected Room room = world.getFirstRoom();
	protected Storage inventory = new Storage("inventory");


	public Game(InputStream is)
	{
		super(is);
	}

	public Room
	getRoom()
	{
		return room;
	}

	public void
	status()
	{
		System.out.println("");
		System.out.println(room.getName() + ".");
	}

	public void
	look(String object_name)
	{
		GameObj target = null;
		if (object_name == null)
			target = room;
		else {
			target = inventory.get(object_name);
			if (target == null)
				target = room.get(object_name);
		}

		if (target == null)
			System.out.println("You can't see any.");
		else
			System.out.println(target.getDescription() + "\n");
	}

	public void
	go(Direction d)
	{
		Room next_room = room.getExit(d);
		if (next_room == null) {
			System.out.println("There is no exit in that direction.");
		} else {
			next_room = room.tryExit(d);
			if (next_room != null) {
				next_room.enterRoom();
				room = next_room;
			}
		}
	}

	public void
	get(String obj_name)
	{
		Item item = room.get(obj_name);
		if (item == null)
			System.out.println("You can't see any\n");
		else {
			item = item.pickUp();
			if (item != null) {
				inventory.put(item);
				System.out.println("Taken.");
			}
		}
	}

	public void
	drop(String obj_name)
	{
		Item item = inventory.get(obj_name);
		if (item == null)
			System.out.println("You don't have it.");
		else {
			room.put(item);
			System.out.println("Dropped.");
		}
	}

	public void
	inventory()
	{
		System.out.println("You are carrying: " + inventory.getContentsNames());
	}

	public void
	use(String obj_name)
	{
		Item item = inventory.get(obj_name);
		if (item == null)
			System.out.println("You don't have it.");
		else item.use();
	}
}
