/***************************************************************************
 Copyright (C) 2007 Christoph Reichenbach <reichenb@colorado.edu>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public Licence as
 published by the Free Software Foundaton; either version 2 of the
 Licence, or (at your option) any later version.

 It is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 merchantability or fitness for a particular purpose. See the
 GNU General Public Licence for more details.

 You should have received a copy of the GNU General Public Licence
 along with this program; see the file COPYING. If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.

***************************************************************************/

package edu.colorado.cs;

public class GameWorld
{
	private Room first_room;
	public Item birdcage;
	public Item fountain;
	public Item pool;
	public Item salt;
	public Wand wand;

	public GameWorld(ReaderInterface reader, RoomReader room_reader)
	{
		first_room = new Room("Entrance", "This is the entrance to the CSCI3155 dungeon.\nRumour has it that untold type errors and polymorphisms lie ahead, as well as the fabled Chamber of Continuations.\n");
		Room sitting_room = new Room ("Sitting room", "Chairs are spread out along the northern walls, and an inviting couch rests in the south.\n"
					      + "It is unclear why they are here, though, as you cannot make out anything worth waiting for.");
		first_room.setExit(Direction.EAST, sitting_room);
		Room fountain_room = new Room ("Fountain room", "A fountain splashes away in the centre of this room, apparently oblivious of your presence.\nIn the ceiling, you notice a shaft leading upwards.  This appears to be the light source for this room and the adjacent rooms.");
		sitting_room.setExit(Direction.EAST, fountain_room);
		Room rubble_room = new Room ("Rubble room", "Part of the ceiling of this room has broken down, leaving rubble and rocks all over the place.");
		fountain_room.setExit(Direction.NORTH, rubble_room);
		Room pool_room = new Room ("Pool", "An underground pool stretches far away to the south.");
		fountain_room.setExit(Direction.SOUTH, pool_room);
		Room goal_room = new GoalRoom("Chamber of Continuations", "As you enter the room beyond, you cannot help but hold your breath in amazement.\n"
					      + "Algebraic datatypes stretch from wall to wall, BNF grammars adorn the ceiling, and the smell of Denotational Semantics is in the air.\n"
					      + "This must be it-- the legendary Chamber of Continuations!\n"
					      + "As you recurse across the room, you cannot help but marvel at the ingenious types the programmers before you have left in the walls.\n"
					      + "\n  Finally, you reach the other end of the room.  There it is, the Altar of Reuse, and the cup glowing on top of it-- undoubtedly--\n"
					      + "this must be the Chalice of Polymorphism, the mythical artifact that is rumoured to bring eternal code re-use to those who code\n"
					      + "with it.  Finally, you have found it!  Drunk with joy, you pick it up.  Now you can return to your laptop and see whether the\n"
					      + "rumours of its incredible powers are true!\n\n" );
		Room ice_room = new IceRoom (Direction.EAST);
		PortalRoom portal_room = new PortalRoom(Direction.NORTH, goal_room, room_reader);
		fountain_room.setExit(Direction.EAST, ice_room);
		ice_room.setExit(Direction.EAST, portal_room);

		birdcage = new Item(rubble_room, "birdcage", "A golden bird cage.  It looks rather useless in this dungeon.");
		fountain = new ImmobileItem(fountain_room, "fountain", "The fountain is occupied spitting water towards the ceiling.  Better not disturb it.") {
				public Item pickUp()
				{
					System.out.println("And get yourself all wet?  I think not.");
					return null;
				}
			};
		pool = new LethalItem(pool_room,
				      "pool",
				      "The surface of the pool is opaque and undisturbed.  You have the eerie feeling that it is watching you.",
				      "As you bend down to pick up the pool, you notice that it is made entirely out of water!\n"
				      + "In your astonishment, you stumble and fall into it, generating a magnificent splash.\n"
				      + "Fortunately, you are too busy drowning to realise what a silly death this was.\n");

		salt = new IceMelterItem(rubble_room, "salt", "It's just a handful of salt.", "You spread the salt on the ice.  Gradually, the ice begins to melt.");
		wand = new Wand(pool_room, reader);
		wand.setMagicSubjects(new MagicInterface[] { wand, portal_room,
							     new MagicAlias("portal", portal_room),
							     new MagicAlias("chalk", portal_room) });
	}

	public Room
	getFirstRoom()
	{
		return first_room;
	}
}
