/***************************************************************************
 Copyright (C) 2007 Christoph Reichenbach <reichenb@colorado.edu>

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public Licence as
 published by the Free Software Foundaton; either version 2 of the
 Licence, or (at your option) any later version.

 It is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of
 merchantability or fitness for a particular purpose. See the
 GNU General Public Licence for more details.

 You should have received a copy of the GNU General Public Licence
 along with this program; see the file COPYING. If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 Boston, MA 02111-1307, USA.

***************************************************************************/

package edu.colorado.cs;

import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;

public class Room extends GameObj implements StorageInterface
{
	private Map<Direction, Room> exits = new HashMap<Direction, Room>();
	private Storage storage;

	public Room(String name, String description)
	{
		super(name, description);
		storage = new Storage(name);
	}

	/**
	 * Determines the presence of an adjacent room
	 */
	public Room
	getExit(Direction d)
	{
		return exits.get(d);
	}


	/**
	 *  Has the player try to exit towards that direction
	 */
	public Room
	tryExit(Direction d)
	{
		return getExit(d);
	}

	public String
	getDescription()
	{
		String description = super.getDescription() + "\n\n";
		Set<Direction> exit_set = exits.keySet();

		/* Some additional fanciness to show the room */
		description += Utility.concatWith("You see no exits",
						  "The only exit is to the ",
						  "Exits are to the ",
						  exit_set);
		description += ".";

		if (!this.storage.isEmpty())
			description += "\n\nYou see here: " + this.getContentsNames();

		return description;
	}


	/**
	 * Sets the exit without ensuring that the other room has a path leading back
	 *
	 * @param d The direction for which the exit ought to be set
	 * @param r The room to point to
	 */
	public void
	forceExit(Direction d, Room r)
	{
		exits.put(d, r);
	}

	/**
	 * Sets the exit of a room and ensures that the opposite direction points to the current room.
	 *
	 * @param d The direction for which the exit ought to be set
	 * @param r The room to point to
	 */
	public void
	setExit(Direction d, Room r)
	{
		this.forceExit(d, r);
		r.forceExit(d.reverse(), this);
	}

	/* Called every time we enter a room */
	public void 
	enterRoom()
	{
	}

	/* Storage interface: Delegation */
	public Item
	get(String s)
	{
		return this.storage.get(s);
	}

	public void
	put(Item i)
	{
		this.storage.put(i);
	}

	public boolean
	isEmpty()
	{
		return this.storage.isEmpty();
	}

	public String
	getContentsNames()
	{
		return this.storage.getContentsNames();
	}

	public Storage
	getStorage()
	{
		return this.storage;
	}
}
